import 'dart:html' as html;
import 'dart:math' as math;
import 'package:stagexl/stagexl.dart';
import 'package:stagexl_particle/stagexl_particle.dart';

ResourceManager resourceManager;
ResourceManager delayedResourceManager;
Bitmap showtimesBitmap;
Bitmap harryBitmap;
Bitmap voldemortBitmap;
Sprite voldemortSprite;
math.Random random;
Stage stage;
RenderLoop renderLoop;
TextField showPrompt;
ParticleEmitter particleEmitter;
var canvas;
GlassPlate glassPlate;

void main() {
    
  canvas = html.query('#stage');
  stage = new Stage('myStage', canvas);
  stage.scaleMode = StageScaleMode.NO_SCALE;
  stage.align = StageAlign.TOP_LEFT;
  renderLoop = new RenderLoop();
  renderLoop.addStage(stage);
  random = new math.Random();
  
  resourceManager = new ResourceManager()
    ..addBitmapData("harry", "img/harry.png")
    ..addBitmapData("voldemort", "img/voldemort.png");
    
  delayedResourceManager = new ResourceManager()
    ..addBitmapData("showtimes", "img/showtimes.png");

  resourceManager.load().then((_) {
    harryBitmap = new Bitmap(resourceManager.getBitmapData("harry"));
    voldemortBitmap = new Bitmap(resourceManager.getBitmapData("voldemort"));
    voldemortSprite = new Sprite();
    voldemortSprite.addChild(voldemortBitmap);
    print(canvas.clientWidth);
    voldemortSprite.x = canvas.clientWidth -(voldemortSprite.width+5);
    
    if (Multitouch.supportsTouchEvents) {
      voldemortSprite.onTouchOver.listen(onVoldemortOver);
    } else {
      voldemortSprite.onMouseOver.listen(onVoldemortOver);
    }

    stage.addChild(voldemortSprite);
    stage.addChild(harryBitmap);
    addText();
    
    delayedResourceManager.load().then((_) {
      showtimesBitmap = new Bitmap(delayedResourceManager.getBitmapData("showtimes"));
    });
  });
    

  particleEmitter = new ParticleEmitter(newAngle(180.0, 200.0));

  stage.addChild(particleEmitter);
  stage.juggler.add(particleEmitter);

  //-------------------------

  /*
   # point 1
x1 = 0.5
y1 = 0.0

# point 2
x2 = -0.5
y2 = -1.0

deltax = x2 - x1
deltay = y2 - y1

angle_rad = atan2(deltay,deltax)
angle_deg = angle_rad*180.0/pi
*/
  

 glassPlate = new GlassPlate(canvas.clientWidth, 600);
 if (Multitouch.supportsTouchEvents) {
    Multitouch.inputMode = MultitouchInputMode.TOUCH_POINT;
    registerTouch();
 } else {
    registerMouse();
 }
  
  stage.addChild(glassPlate);
}

void registerTouch() {
  var touchEventListener = (te) {
    var deltax = te.localX - 203;
    var deltay = te.localY - 111;
    num angle_rad = math.atan2(deltay,deltax);
    
    particleEmitter.updateConfig(newAngle((angle_rad*180.0/math.PI)+180, te.localX));
    (te as TouchEvent).stopImmediatePropagation();
  };
  glassPlate.onTouchBegin.listen(touchEventListener);
  glassPlate.onTouchMove.listen(touchEventListener);
}

void registerMouse() {
  var mouseEventListener = (MouseEvent me) {
    //if (me.buttonDown) {
      //showPrompt.text = "mouse: " + random.nextInt(1000).toString();
      var deltax = me.localX - 203;
      var deltay = me.localY - 111;
      num angle_rad = math.atan2(deltay,deltax);
      
      particleEmitter.updateConfig(newAngle((angle_rad*180.0/math.PI)+180, me.localX));
      me.stopImmediatePropagation();

      //particleEmitter.setEmitterLocation(me.localX, me.localY);
    //}
  };
  glassPlate.onMouseDown.listen(mouseEventListener);
  glassPlate.onMouseMove.listen(mouseEventListener);
}

Map newAngle(double newAngle, double xVal) {
  double length = xVal - 203.0;
  if(length<50.0) length=50.0;
  
  return  {
      "maxParticles": 200,
      "duration": 0,
      "lifeSpan": 0.9,
      "lifespanVariance": 0.4,
      "startSize": 10,
      "startSizeVariance": 20,
      "finishSize": 70,
      "finishSizeVariance": 0,
      "shape": "circle",
      "emitterType": 1,
      "location": {
        "x": 203,
        "y": 111
      },
      "locationVariance": {
        "x": 0,
        "y": 0
      },
      "speed": 100,
      "speedVariance": 10,
      "angle": newAngle,
      "angleVariance": 5,
      "gravity": {
        "x": 0,
        "y": 100
      },
      "radialAcceleration": 20,
      "radialAccelerationVariance": 0,
      "tangentialAcceleration": 10,
      "tangentialAccelerationVariance": 0,
      "minRadius": 0,
      "maxRadius": (length * 3/5) + 30,
      "maxRadiusVariance": (length * 2/5),
      "rotatePerSecond": 0,
      "rotatePerSecondVariance": 5,
      "compositeOperation": "source-over",
      "startColor": {
        "red": 1,
        "green": 0.7,
        "blue": 0.07,
        "alpha": 1
      },
      "finishColor": {
        "red": 0.5 + (random.nextDouble()*0.5),
        "green": 0.1 + (random.nextDouble()*0.9),
        "blue": 0.1 + (random.nextDouble()*0.9),
        "alpha": 0
      }
  };
}

void onVoldemortOver(e) {
  var deltax = e.stageX - 203;
  var deltay = e.stageY - 111;
  num angle_rad = math.atan2(deltay,deltax);
  particleEmitter.updateConfig(newAngle((angle_rad*180.0/math.PI)+180, e.stageX));
  
  var tween = new Tween(voldemortSprite, 1.3, TransitionFunction.easeOutBounce);
  tween.animate.x.to(voldemortSprite.x + voldemortSprite.width/2);
  tween.animate.y.to(voldemortSprite.y + voldemortSprite.height/2);
  tween.animate.scaleX.to(0);
  tween.animate.scaleY.to(0);
  tween.onComplete = () => onVoldemortComplete();
  renderLoop.juggler.add(tween);
 
  var harryOutTween = new Tween(harryBitmap, 1.6, TransitionFunction.easeInExponential);
  harryOutTween.animate.alpha.to(0);
  renderLoop.juggler.add(harryOutTween);
  
  showtimesBitmap.alpha = 0;
  stage.addChild(showtimesBitmap);
  var showTween = new Tween(showtimesBitmap, 2, TransitionFunction.easeInExponential);
  showTween.animate.alpha.to(1);
  renderLoop.juggler.add(showTween);
}

void onVoldemortComplete() {
  voldemortSprite.removeFromParent();
  particleEmitter.removeFromParent();
  //tween these ones out as showtimes come in
  harryBitmap.removeFromParent();
  showPrompt.removeFromParent();
  //show showtimes;
}

void addText() {
  showPrompt = new TextField();
  showPrompt.defaultTextFormat = new TextFormat('Spicy Rice', 30, Color.DarkCyan, align:TextFormatAlign.CENTER);
  showPrompt.text = 'Zap Voldemort for Showtimes';
  showPrompt.autoSize = TextFieldAutoSize.CENTER;
  showPrompt.x = harryBitmap.x + harryBitmap.width + 20;
  showPrompt.y = 50;
  showPrompt.width = canvas.clientWidth - (harryBitmap.x + harryBitmap.width + 20) - (voldemortBitmap.width + 20);
  showPrompt.height = 200;
  showPrompt.wordWrap = true;
  showPrompt.mouseEnabled = false;
  stage.addChild(showPrompt);
}